/* 
   Matrix multiplication example

   OpenMP version

   Jim Teresco, CS 338, Williams College
   Mon Feb 10 10:55:31 EST 2003

   $Id$
*/

/* header files needed for printf, gettimeofday, struct timeval */
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <omp.h>

/* header file for our own timer.c function diffgettime */
#include "timer.h"

/* we will multiply square matrices, how big? */
#define SIZE 750

/* it's a simple program for now, we'll just put everything in main */
int main(int argc, char *argv[]) {

  /* our matrices */
  /*  double a[SIZE][SIZE], b[SIZE][SIZE], c[SIZE][SIZE]; */
  double **a, **b, **c; 

  /* counters */
  int i, j, k;
  int sum;

  /* to pass to gettimeofday to get wall clock times */
  struct timeval start, stop;

  /* initialize and allocate matrices, just fill with junk */
  gettimeofday(&start, NULL);
  a = (double **)malloc(SIZE*sizeof(double *));
  b = (double **)malloc(SIZE*sizeof(double *));
  c = (double **)malloc(SIZE*sizeof(double *));
  for (i=0; i<SIZE; i++) {
    a[i] = (double *)malloc(SIZE*sizeof(double));
    b[i] = (double *)malloc(SIZE*sizeof(double));
    c[i] = (double *)malloc(SIZE*sizeof(double));
    for (j=0; j<SIZE; j++) {
      a[i][j] = i+j;
      b[i][j] = i-j;
    }
  }
  gettimeofday(&stop, NULL);
  printf("Initialization took: %f seconds\n", diffgettime(start,stop));
  
  gettimeofday(&start, NULL);
  /* matrix-matrix multiply */
#pragma omp parallel for
  for (i=0; i<SIZE; i++) {  /* for each row */
    for (j=0; j<SIZE; j++) { /* for each column */
      /* initialize result to 0 */
      c[i][j] = 0;

      /* perform dot product */
      for(k=0; k<SIZE; k++) {
	c[i][j] = c[i][j] + a[i][k]*b[k][j];
      }
    }
  }
  gettimeofday(&stop, NULL);
  printf("Multiplication took: %f seconds\n", diffgettime(start,stop));
  
  /* This is here to make sure the optimizing compiler doesn't
     get any big ideas about "optimizing" code away completely */
  sum=0;
  for (i=0; i<SIZE; i++) {
    for (j=0; j<SIZE; j++) {
      sum += c[i][j];
    }
    free(a[i]);
    free(b[i]);
    free(c[i]);
  }
  free(a);
  free(b);
  free(c);
  printf("Sum of elements of c=%f\n", sum);

  return 0;
}
